/* EPS Conduits (C) 2014-2017 Stephane Charette <stephanecharette@gmail.com>
 * $Id: logging.hpp 2167 2017-03-01 12:10:04Z stephane $
 * See license.txt.
 */

#pragma once

#include <boost/log/trivial.hpp>
#include "common.hpp"


namespace EPS
{
	/// Initialize boost::log for %EPS client/server.
	void initialize_logging(const EPS::EMode mode);

	/// Add the specified filename to the logging output.
	void initialize_file_logging(const std::string &filename);

	/// Reset boost::log and disable logging.
	void reset_logging(void);

	/// Log a line for every function name in the call stack.
	void log_backtrace(void);

	/// All logging should be done using one of the %EPS LOG macros. @{
	#define LOG_TRACE	BOOST_LOG_TRIVIAL(trace)
	#define LOG_DEBUG	BOOST_LOG_TRIVIAL(debug)
	#define LOG_INFO	BOOST_LOG_TRIVIAL(info)
	#define LOG_WARN	BOOST_LOG_TRIVIAL(warning)
	#define LOG_ERROR	BOOST_LOG_TRIVIAL(error)
	#define LOG_FATAL	BOOST_LOG_TRIVIAL(fatal)
	/// @}

	/// Macros to both log and throw std::exception objects. @{
	#define THROW_RUNTIME_ERROR(error_no, msg)																				\
					log_backtrace();																							\
					LOG_ERROR << "Error in " << __PRETTY_FUNCTION__ << ": errno " << error_no << ": " << strerror(error_no);	\
					LOG_ERROR << "ERROR!  " << msg;																				\
					LOG_ERROR << "ERROR!  Throwing std::runtime_error due to previous error.";									\
					throw std::runtime_error(msg);

	#define THROW_INVALID_ARGUMENT(error_no, msg)																			\
					log_backtrace();																							\
					LOG_ERROR << "Error in " << __PRETTY_FUNCTION__ << ": errno " << error_no << ": " << strerror(error_no);	\
					LOG_ERROR << "ERROR!  " << msg;																				\
					LOG_ERROR << "ERROR!  Throwing std::invalid_argument due to previous error.";								\
					throw std::invalid_argument(msg);
	/// @}
}
